package ru.dbotthepony.kstarbound.client.render

import org.lwjgl.glfw.GLFW.glfwGetTime

/**
 * Таймер для анимирования набора спрайтов
 */
open class FrameAnimator(
	/**
	 * Первый кадр в анимации
	 */
	var firstFrame: Int = 0,

	/**
	 * Последний кадр в анимации
	 */
	var lastFrame: Int,

	/**
	 * Сколько времени занимает один кадр
	 */
	var animationCycle: Double,

	/**
	 * Зациклить ли анимацию
	 */
	var animationLoops: Boolean = true,
) {
	var frame = 0
		private set

	/**
	 * Возвращает разницу между последним и первым кадром анимации
	 */
	val frameDiff get() = lastFrame - firstFrame

	private val initial = glfwGetTime()
	private var lastRender = initial

	/**
	 * Сколько времени прошло с момента последнего кадра
	 */
	val delta get() = glfwGetTime() - lastRender

	private var counter = 0.0

	/**
	 * Проверяет glfw таймер и продвигает фрейм анимации
	 */
	fun advance() {
		if (frameDiff == 0)
			return

		if (frame + firstFrame >= lastFrame && !animationLoops) {
			return
		}

		counter += delta / animationCycle
		lastRender = glfwGetTime()

		if (counter >= 1.0) {
			val desired = frame + counter.toInt()

			if (animationLoops || desired + firstFrame < lastFrame) {
				frame = desired % frameDiff
				counter %= 1.0
			} else {
				frame = lastFrame - firstFrame
			}
		}
	}
}
