package ru.dbotthepony.kstarbound.client.render

import com.google.common.collect.ImmutableList
import ru.dbotthepony.kstarbound.client.gl.GLStateTracker
import ru.dbotthepony.kstarbound.client.gl.GLTexture2D
import ru.dbotthepony.kstarbound.defs.animation.AtlasConfiguration
import ru.dbotthepony.kstarbound.defs.animation.ImageReference

class SpriteAnimator(
	val sprites: List<BoundSprite>,
	animationCycle: Double,
	animationLoops: Boolean = true,
	firstFrame: Int = 0,
	lastFrame: Int = sprites.size - 1
) : FrameAnimator(
	firstFrame = firstFrame,
	lastFrame = lastFrame,
	animationCycle = animationCycle,
	animationLoops = animationLoops,
) {
	constructor(
		image: GLTexture2D,
		atlas: AtlasConfiguration,
		animationCycle: Double,
		animationLoops: Boolean = true,
		firstFrame: Int = 0,
		lastFrame: Int = atlas.spriteList.size - 1
	) : this(
		atlas.spriteList.stream().map { it.bind(image) }.collect(ImmutableList.toImmutableList()),
		animationCycle = animationCycle,
		animationLoops = animationLoops,
		firstFrame = firstFrame,
		lastFrame = lastFrame,
	)

	constructor(
		image: ImageReference,
		state: GLStateTracker,
		animationCycle: Double,
		animationLoops: Boolean = true,
		firstFrame: Int = 0,
		lastFrame: Int = image.config.spriteList.size - 1
	) : this(state.loadNamedTextureSafe(image.image), image.config, animationCycle, animationLoops, firstFrame, lastFrame)

	val sprite get() = sprites[frame]
}

fun ImageReference.makeSpriteAnimator(
	state: GLStateTracker,
	animationCycle: Double,
	animationLoops: Boolean = true,
	firstFrame: Int = 0,
	lastFrame: Int = config.spriteList.size - 1
): SpriteAnimator {
	return SpriteAnimator(this, state, animationCycle, animationLoops, firstFrame, lastFrame)
}
