
#version 460

// Huge thanks to articles by Scott [slembcke] Lembcke!
// https://slembcke.github.io/SuperFastHardShadows
// https://slembcke.github.io/SuperFastSoftShadows

layout (location = 0) in vec4 packedVertexPos;
layout (location = 1) in vec2 edgeType;

uniform mat4 transform;
uniform mat4 localToWorldTransform;

uniform vec3 lightPositionAndSize;
uniform float lightPenetration;

varying vec4 v_penumbras;
varying vec3 v_edges;
varying vec3 v_proj_pos;
varying vec4 v_endpoints;

// Keep in mind GLSL is column major. You'll need to swap row/column for HLSL.
mat2 adjugate(mat2 m) {
	return mat2(m[1][1], -m[0][1], -m[1][0], m[0][0]);
}

void main() {
	vec2 lightPosition = lightPositionAndSize.xy;
	float lightSize = lightPositionAndSize.z;

	// Unpack the vertex shader input.
	vec2 endpoint_a = (localToWorldTransform * vec4(packedVertexPos.zw, 0.0, 1.0)).xy;
	vec2 endpoint_b = (localToWorldTransform * vec4(packedVertexPos.xy, 0.0, 1.0)).xy;
	vec2 endpoint = mix(endpoint_a, endpoint_b, edgeType.x);

	// Deltas from the segment to the light center.
	vec2 delta_a = endpoint_a - lightPosition;
	vec2 delta_b = endpoint_b - lightPosition;
	vec2 delta = endpoint - lightPosition;

	// Offsets from the light center to the edge of the light volume.
	vec2 offset_a = vec2(-lightSize,  lightSize) * normalize(delta_a).yx;
	vec2 offset_b = vec2( lightSize, -lightSize) * normalize(delta_b).yx;
	vec2 offset = mix(offset_a, offset_b, edgeType.x);

	// Vertex projection.
	float w = edgeType.y;
	vec4 proj_pos = vec4(mix(delta - offset, endpoint, w), 0.0, w);

	gl_Position = transform * proj_pos;

	vec2 penumbra_a = adjugate(mat2( offset_a, -delta_a))*(delta - mix(offset, delta_a, w));
	vec2 penumbra_b = adjugate(mat2(-offset_b,  delta_b))*(delta - mix(offset, delta_b, w));

	v_penumbras = (lightSize > 0.0 ? vec4(penumbra_a, penumbra_b) : vec4(0, 1, 0, 1));

	// Edge values for light penetration and clipping.
	vec2 seg_delta = endpoint_b - endpoint_a;
	vec2 seg_normal = seg_delta.yx*vec2(-1.0, 1.0);
	// Calculate where the light -> pixel ray will intersect with the segment.
	v_edges.xy = -adjugate(mat2(seg_delta, delta_a + delta_b))*(delta - offset*(1.0 - w));
	v_edges.y *= 2.0; // Skip a multiply in the fragment shader.
	// Calculate a clipping coordinate that is 0 at the near edge (when w = 1)...
	// otherwise calculate the dot product with the projected coordinate.
	v_edges.z = dot(seg_normal, delta - offset)*(1.0 - w);

	// Light penetration values.
	v_proj_pos = vec3(proj_pos.xy, w * lightPenetration);
	v_endpoints = vec4(endpoint_a, endpoint_b) / lightPenetration;
}
